'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var coffee = _interopDefault(require('coffeescript'));
var path = _interopDefault(require('path'));
var fs = _interopDefault(require('fs'));
var acorn = require('acorn');
var estraverse = _interopDefault(require('estraverse'));
var escodegen = _interopDefault(require('escodegen'));

var formatError = function (source, filename, err) {
    var carets, col, first_column, first_line, l, last_column, last_line, lines;
    ({first_line, last_line, first_column, last_column} = err.location);
    if (last_line == null) {
        last_line = first_line;
    }
    lines = source.split('\n');
    l = lines[first_line];
    lines[first_line] = l.substring(0, first_column) + '\x1B[91m' + l.substring(first_column);
    l = lines[last_line];
    if (first_line === last_line) {
        col = last_column + 6;
    } else {
        col = last_column;
    }
    lines[last_line] = l.substring(0, col) + '\x1B[39m' + l.substring(col);
    lines = lines.slice(first_line, last_line + 1);
    carets = Array(first_column + 1).join(' ') + `\x1B[91m${ Array(last_column + 2 - first_column).join('^') }\x1B[39m`;
    lines.push(carets);
    return `${ filename }:${ first_line + 1 }:${ first_column + 1 }: ${ err.name }: ${ err.message }\n${ lines.join('\n') }\n`;
};

var ASTToCode;
var getAST;
var rCFPath;
var ricffAST;
var _extends = Object.assign || function (target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }
        return target;
    };
rCFPath = filePath => {
    var fileObj;
    fileObj = _extends({ exists: fs.existsSync(filePath) }, path.parse(filePath));
    if (!fileObj.exists) {
        if (fileObj.ext === '.js') {
            return filePath;
        }
        if (fileObj.ext === '.coffee') {
            return rCFPath(path.format({
                root: '/ignored',
                dir: fileObj.dir,
                base: fileObj.base,
                name: 'ignored',
                ext: '.js'
            }));
        } else {
            return rCFPath(`${ filePath }.coffee`);
        }
    } else {
        if (fileObj.ext !== '') {
            return filePath;
        } else {
            return rCFPath(path.join(filePath, 'index.coffee'));
        }
    }
};
ricffAST = (ast, dirname = __dirname) => {
    return estraverse.replace(ast, {
        leave: function (currentNode, parentNode) {
            var fileParseObj, filePath;
            if (currentNode.type === 'ImportDeclaration') {
                fileParseObj = path.parse(currentNode.source.value);
                if (fileParseObj.root === '' && fileParseObj.dir === '') {
                    return currentNode;
                }
                filePath = rCFPath(path.join(dirname, currentNode.source.value));
                currentNode.source.value = filePath;
            }
            return currentNode;
        }
    });
};
getAST = source => {
    var comments, tokens;
    comments = [];
    tokens = [];
    return acorn.parse(source, {
        sourceType: 'module',
        ranges: true,
        onComment: comments,
        Token: tokens
    });
};
ASTToCode = function (ast) {
    return escodegen.generate(ast);
};

var sourceMap;
sourceMap = function (out) {
    if (out.v3SourceMap) {
        return JSON.parse(out.v3SourceMap);
    } else {
        return null;
    }
};
var coffee2$1 = function (opts = {}) {
    if (opts.bare == null) {
        opts.bare = true;
    }
    if (opts.sourceMap == null) {
        opts.sourceMap = true;
    }
    return {
        name: 'coffee2',
        transform: function (code, id) {
            var err, out;
            if (path.extname(id) !== '.coffee') {
                return { code };
            }
            try {
                out = coffee.compile(code, {
                    filename: id,
                    bare: opts.bare,
                    sourceMap: opts.sourceMap
                });
            } catch (error) {
                err = error;
                if (err.location != null) {
                    err.formattedMessage = formatError(code, id, err);
                }
                throw err;
            }
            return {
                code: ASTToCode(ricffAST(getAST(out.js), path.dirname(id))),
                map: sourceMap(out)
            };
        }
    };
};

module.exports = coffee2$1;
